/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/

#include <iostream>
#include <algorithm>
//Some doc at : https://en.cppreference.com/w/cpp/header/execution
#include <execution>

#include "asterics_hpc.h"

///Do the Hadamard product
/**	@param[out] ptabResult : table of results of tabX*tabY
 * 	@param ptabX : input table
 * 	@param ptabY : input table
 * 	@param nbElement : number of elements in the tables
*/
void hadamard_product(float* __restrict__ ptabResult, const float* __restrict__ ptabX, const float* __restrict__ ptabY, long unsigned int nbElement){
	std::transform(std::execution::unseq, ptabX, ptabX+nbElement, ptabY, ptabResult,
			[](float xi, float yi){ return xi * yi; });
}

///Get the number of cycles per elements of the Hadamard product
/**	@param nbElement : number of elements of the tables
 * 	@param nbRepetition : number of repetition to evaluate the function hadamard_product
*/
void evaluateHadamardProduct(long unsigned int nbElement, long unsigned int nbRepetition){
	float * tabResult = (float*)asterics_malloc(sizeof(float)*nbElement);
	float * tabX = (float*)asterics_malloc(sizeof(float)*nbElement);
	float * tabY = (float*)asterics_malloc(sizeof(float)*nbElement);
	
	for(long unsigned int i(0lu); i < nbElement; ++i){
		tabX[i] = (float)(i*32lu%17lu);
		tabY[i] = (float)(i*57lu%31lu);
	}
	float res(0.0f);
	long unsigned int beginTime(rdtsc());
	for(long unsigned int i(0lu); i < nbRepetition; ++i){
		hadamard_product(tabResult, tabX, tabY, nbElement);
		res += tabResult[0];
	}
	long unsigned int elapsedTime((double)(rdtsc() - beginTime)/((double)nbRepetition));
	
	double cyclePerElement(((double)elapsedTime)/((double)nbElement));
	std::cout << "evaluateHadamardProduct : nbElement = "<<nbElement<<", cyclePerElement = " << cyclePerElement << " cy/el, elapsedTime = " << elapsedTime << " cy, res = " << res << std::endl;
	std::cerr << nbElement << "\t" << cyclePerElement << "\t" << elapsedTime << std::endl;
	asterics_free(tabResult);
	asterics_free(tabX);
	asterics_free(tabY);
}

int main(int argc, char** argv){
	std::cout << "Hadamard product vectorized" << std::endl;
	evaluateHadamardProduct(1000lu, 1000000lu);
	evaluateHadamardProduct(1500lu, 1000000lu);
	evaluateHadamardProduct(2000lu, 1000000lu);
	evaluateHadamardProduct(2500lu, 1000000lu);
	evaluateHadamardProduct(2666lu, 1000000lu);
	evaluateHadamardProduct(3000lu, 1000000lu);
	evaluateHadamardProduct(4000lu, 1000000lu);
	evaluateHadamardProduct(5000lu, 1000000lu);
	evaluateHadamardProduct(10000lu, 1000000lu);
	return 0;
}
